@@ -10,22 +10,10 @@
  *
  * Maintainer: Gennady Feldman <gfeldman@gena01.com> as of Mar 12, 2001
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+ * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
+#include "busybox.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -45,8 +33,6 @@
 #include <sys/un.h>
 #include <sys/param.h>
 
-#include "busybox.h"
-
 /* SYSLOG_NAMES defined to pull some extra junk from syslog.h */
 #define SYSLOG_NAMES
 #include <sys/syslog.h>
@@ -222,7 +208,7 @@ static void circ_message(const char *msg)
 	 *      "tail" are actually offsets from the beginning of the buffer.
 	 *
 	 * Note: This algorithm uses Linux IPC mechanism w/ shared memory and semaphores to provide
-	 *       a threasafe way of handling shared memory operations.
+	 *       a threadsafe way of handling shared memory operations.
 	 */
 	if ((buf->tail + l) < buf->size) {
 		/* before we append the message we need to check the HEAD so that we won't
@@ -249,7 +235,7 @@ static void circ_message(const char *msg)
 					/* Note: HEAD is only used to "retrieve" messages, it's not used
 					   when writing messages into our buffer */
 				} else {	/* show an error message to know we messed up? */
-					printf("Weird! Can't find the terminator token??? \n");
+					printf("Weird! Can't find the terminator token?\n");
 					buf->head = 0;
 				}
 			}
@@ -286,7 +272,7 @@ static void circ_message(const char *msg)
 			buf->tail = k + 1;
 		} else {
 			printf
-				("Weird! Can't find the terminator token from the beginning??? \n");
+				("Weird! Can't find the terminator token from the beginning?\n");
 			buf->head = buf->tail = 0;	/* reset buffer, since it's probably corrupted */
 		}
 
@@ -382,12 +368,7 @@ static void message(char *fmt, ...)
 static void init_RemoteLog(void)
 {
 	memset(&remoteaddr, 0, sizeof(remoteaddr));
-	remotefd = socket(AF_INET, SOCK_DGRAM, 0);
-
-	if (remotefd < 0) {
-		bb_error_msg("cannot create socket");
-	}
-
+	remotefd = bb_xsocket(AF_INET, SOCK_DGRAM, 0);
 	remoteaddr.sin_family = AF_INET;
 	remoteaddr.sin_addr = *(struct in_addr *) *(xgethostbyname(RemoteHost))->h_addr_list;
 	remoteaddr.sin_port = htons(RemotePort);
@@ -399,7 +380,7 @@ static void logMessage(int pri, char *msg)
 	time_t now;
 	char *timestamp;
 	static char res[20];
-#ifdef CONFIG_FEATURE_REMOTE_LOG	
+#ifdef CONFIG_FEATURE_REMOTE_LOG
 	static char line[MAXLINE + 1];
 #endif
 	CODE *c_pri, *c_fac;
@@ -527,7 +508,7 @@ static int serveConnection(char *tmpbuf, int n_read)
 	return n_read;
 }
 
-static void doSyslogd(void) __attribute__ ((noreturn));
+static void doSyslogd(void) ATTRIBUTE_NORETURN;
 static void doSyslogd(void)
 {
 	struct sockaddr_un sunx;
@@ -556,11 +537,7 @@ static void doSyslogd(void)
 	memset(&sunx, 0, sizeof(sunx));
 	sunx.sun_family = AF_UNIX;
 	strncpy(sunx.sun_path, lfile, sizeof(sunx.sun_path));
-	if ((sock_fd = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0) {
-		bb_perror_msg_and_die("Couldn't get file descriptor for socket "
-						   _PATH_LOG);
-	}
-
+	sock_fd = bb_xsocket(AF_UNIX, SOCK_DGRAM, 0);
 	addrLength = sizeof(sunx.sun_family) + strlen(sunx.sun_path);
 	if (bind(sock_fd, (struct sockaddr *) &sunx, addrLength) < 0) {
 		bb_perror_msg_and_die("Could not connect to socket " _PATH_LOG);
@@ -581,7 +558,7 @@ static void doSyslogd(void)
 	}
 #endif
 
-	logMessage(LOG_SYSLOG | LOG_INFO, "syslogd started: " BB_BANNER);
+	logMessage(LOG_SYSLOG | LOG_INFO, "syslogd started: " "BusyBox v" BB_VER );
 
 	for (;;) {
 
@@ -615,7 +592,7 @@ static void doSyslogd(void)
 	}					/* for main loop */
 }
 
-extern int syslogd_main(int argc, char **argv)
+int syslogd_main(int argc, char **argv)
 {
 	int opt;
 
@@ -692,22 +669,13 @@ extern int syslogd_main(int argc, char **argv)
 	umask(0);
 
 	if (doFork == TRUE) {
-#if defined(__uClinux__)
+#ifdef BB_NOMMU
 		vfork_daemon_rexec(0, 1, argc, argv, "-n");
-#else /* __uClinux__ */
-		if(daemon(0, 1) < 0)
-			bb_perror_msg_and_die("daemon");
-#endif /* __uClinux__ */
+#else
+		bb_xdaemon(0, 1);
+#endif
 	}
 	doSyslogd();
 
 	return EXIT_SUCCESS;
 }
-
-/*
-Local Variables
-c-file-style: "linux"
-c-basic-offset: 4
-tab-width: 4
-End:
-*/
